import glob
import json
import os
import tempfile
from datetime import date
from unittest import TestCase

from platformdirs import user_data_dir

from directorium.api import Api, CacheApi, FileApi, RequestApi


class TestApi(TestCase):

    def clearCache(self) -> None:
        for f in glob.glob(f"{self.cache_dir}/*.json"):
            os.remove(f)
        if os.path.exists(self.cache_dir):
            os.removedirs(self.cache_dir)

    def setUp(self) -> None:
        self.cache_dir = user_data_dir("directorium", False)
        self.clearCache()

    def tearDown(self) -> None:
        self.clearCache()

    def test_request(self):
        with open("tests/data/2024.json", "r") as f:
            actual = RequestApi().get_year(2024)
            expected = json.load(f)
            self.assertEqual(actual, expected)


    def test_request_calendar(self):
        with open("tests/data/2025.koeln.json", "r") as f:
            api = RequestApi("koeln")
            actual = api.get_year(2025)
            expected = json.load(f)
            self.assertEqual(actual, expected)


    def test_file(self):
        with open("tests/data/2024.json", "r") as f:
            api = FileApi("tests/data/%s.json")
            actual = api.get_year(2024)
            expected = json.load(f)
            self.assertEqual(actual, expected)

    def test_cache(self):
        with (
            tempfile.TemporaryDirectory() as tempdir,
            open("tests/data/2024.json", "r") as f,
        ):
            api = CacheApi(tempdir)
            path = f"{tempdir}/2024.json"
            self.assertFalse(os.path.exists(path))
            actual = api.get_year(2024)
            self.assertTrue(os.path.exists(path))
            expected = json.load(f)
            self.assertEqual(actual, expected)
            with open(path, "r") as f:
                actual = json.load(f)
                self.assertEqual(actual, expected)
            actual = api.get_year(2024)
            self.assertEqual(actual, expected)

    def test_cache_calendar(self):
        with (
            tempfile.TemporaryDirectory() as tempdir,
            open("tests/data/2025.koeln.json", "r") as f,
        ):
            api = CacheApi(tempdir, calendar="koeln")
            path = f"{tempdir}/2025.koeln.json"
            self.assertFalse(os.path.exists(path))
            actual = api.get_year(2025)
            self.assertTrue(os.path.exists(path))
            expected = json.load(f)
            self.assertEqual(actual, expected)
            with open(path, "r") as f:
                actual = json.load(f)
                self.assertEqual(actual, expected)
            actual = api.get_year(2025)
            self.assertEqual(actual, expected)

    def test_cache_without_dir(self):
        self.assertFalse(os.path.exists(self.cache_dir))
        self.assertFalse(os.path.exists(f"{self.cache_dir}/2024.json"))
        CacheApi().get_year(2024)
        self.assertTrue(os.path.exists(self.cache_dir))
        self.assertTrue(os.path.exists(f"{self.cache_dir}/2024.json"))
        with (
            open("tests/data/2024.json", "r") as f,
            open(f"{self.cache_dir}/2024.json", "r") as g,
        ):
            actual = json.load(g)
            expected = json.load(f)
            self.assertEqual(actual, expected)

    def test_get_date(self):
        with open("tests/data/2024.json", "r") as f:
            data = json.load(f)
            expected = data["Zelebrationen"][1:3]

        class TestApi(Api):
            def get_year(self, year):
                with open("tests/data/2024.json", "r") as f:
                    return json.load(f)

        api = TestApi()
        self.assertEqual(api.get_date(2024, 1, 2), expected)
        self.assertEqual(api.get_date(date(2024, 1, 2)), expected)

    def test_request_get_date(self):
        with open("tests/data/2024.json", "r") as f:
            data = json.load(f)
            expected = data["Zelebrationen"][1:3]

        api = RequestApi()
        self.assertEqual(api.get_date(2024, 1, 2), expected)
        self.assertEqual(api.get_date(date(2024, 1, 2)), expected)

    def test_cache_get_date(self):
        with (
            tempfile.TemporaryDirectory() as tempdir,
            open("tests/data/2025.koeln.json", "r") as f,
        ):
            api = CacheApi(tempdir, calendar="koeln")
            path = f"{tempdir}/2025.koeln.json"
            self.assertFalse(os.path.exists(path))
            actual = api.get_date(2025, 1, 2)
            self.assertTrue(os.path.exists(path))
            expected = json.load(f)
            self.assertEqual(actual, expected["Zelebrationen"][1:4])
            with open(path, "r") as f:
                actual = json.load(f)
                self.assertEqual(actual, expected)
            actual = api.get_year(2025)
            self.assertEqual(actual, expected)



"""Actual implementation of the Directorium class.

This module contains the Directorium class which is the main entry point for
the liturgical calendar. It provides methods to get the liturgical events for
a specific date and to determine the season of the liturgical year.
"""

import enum
from datetime import date, timedelta
from typing import List

from . import utils
from .api import Api
from .event import Event


class Season(enum.Enum):
    """The season of the liturgical year."""

    ORDINARY = enum.auto()
    """
    The ordinary season of the liturgical year - if no other season is currently
    active.
    """

    CHRISTMAS = enum.auto()
    """
    The christmas season starts with the first advent sunday and ends with the
    baptism of the lord (i.e. the sunday after epiphany).
    """

    LENT = enum.auto()
    """The lent season starts with ash wednesday and ends before easter sunday."""

    EASTER = enum.auto()
    """Easter season starts with easter sunday and ends with pentecost."""


class Directorium:
    """The main class to access the liturgical calendar.

    The API-backend is exchangeable to allow for different sources of the
    liturgical calendar data.

    Attributes:
        api (Api): The API to get the liturgical events from.
    """

    def __init__(self, api: Api):
        """Creates the Directorium with the given API.

        Args:
            api (Api): The api to get the liturgical events from.
        """
        self.api = api

    def get(self, d: date) -> List[Event]:
        """Retrieves a list of liturgical events for a specific date.

        Args:
            d (date): The date for which to get the liturgical events.

        Returns:
            List[Event]: List of liturgical events for the given date.
        """
        return [Event.parse(e) for e in self.api.get_date(d)]

    def season(self, d: date) -> Season:
        """Determines the season of the liturgical year for a specific date.

        Args:
            d (date): The date for which to determine the season.

        Returns:
            Season: The season of the liturgical year for the given date.
        """
        christmas = date(d.year, 12, 25)
        advent = christmas - timedelta(days=21 + christmas.isoweekday())
        if d >= advent:
            return Season.CHRISTMAS

        epiphany = date(d.year, 1, 6)
        baptism = epiphany + timedelta(days=7 - epiphany.isoweekday() % 7)
        if d <= baptism:
            return Season.CHRISTMAS

        easter = utils.easter(d.year)
        ashwednesday = easter - timedelta(days=46)
        if d >= ashwednesday and d < easter:
            return Season.LENT

        pentecoste = easter + timedelta(days=49)
        if d >= easter and d < pentecoste:
            return Season.EASTER

        return Season.ORDINARY

    @staticmethod
    def from_request(calendar: str | None = None) -> "Directorium":
        """Creates a Directorium instance with a `RequestApi` backend.

        Args:
            calendar (str | None, optional): The name of the calendar to use.

        Returns:
            Directorium: The Directorium instance with a RequestApi backend.
        """
        from .api import RequestApi
        return Directorium(RequestApi(calendar))

    @staticmethod
    def from_file(format_path: str) -> "Directorium":
        """Creates a Directorium instance with a `FileApi` backend.

        Args:
            format_path (str): The format string for the file path.

        Returns:
            Directorium: The Directorium instance with a FileApi backend.
        """
        from .api import FileApi
        return Directorium(FileApi(format_path))

    @staticmethod
    def from_cache(base_path: str | None = None, calendar: str | None = None) -> "Directorium":
        """Creates a Directorium instance with a `CacheApi` backend.

        Args:
            base_path (str | None, optional): The base path for the cache files.
                If None, the default folder defined in `CacheApi` is used.
            calendar (str | None, optional): The name of the calendar to use.

        Returns:
            Directorium: The Directorium instance with a CacheApi backend.
        """
        from .api import CacheApi
        return Directorium(CacheApi(base_path, calendar))



"""Module containing the Event class and associated data types.

The event is a liturgical celebration with a date, title, comment, readings, and
more. It defines the color of the day and the rank of the celebration as an enum
to simplify the handling of the data.
"""

import enum
from dataclasses import dataclass
from datetime import date
from typing import Optional

from .api import ApiEvent


class Color(enum.Enum):
    """The color of the liturgical celebration."""

    NONE = enum.auto()
    WHITE = enum.auto()
    RED = enum.auto()
    GREEN = enum.auto()
    VIOLET = enum.auto()
    ROSE = enum.auto()


class Rank(enum.IntEnum):
    """The rank of a celebration. A higher rank has a higher priority."""

    NONE = 0
    COMMEMORATION = 1
    OPTIONAL_MEMORIAL = 2
    MEMORIAL = 3
    FEAST = 4
    SOLEMNITY = 5


@dataclass
class Event:
    """Smallest data unit of the liturgical calendar.

    An Event contains many information about a liturgical celebration like the
    date, title, comment, readings, color, rank, and more.
    """

    date: date
    """The date of the liturgical celebration."""

    title: str
    """A title for the liturgical celebration."""

    comment: str = ""
    """An additional comment explaining the celebration."""

    lecture1: str = ""
    """First lecture."""

    psalm: str = ""
    """Responsorial psalm."""

    lecture2: str = ""
    """Second lecture."""

    gospel: str = ""
    """The gospel of the day."""

    color: Optional[Color] = None
    """The liturgical color of the celebration."""

    file: str = ""
    """A path of the online shott with more information."""

    importance: int = 0
    """Importance of the celebration. Higher values have a higher priority."""

    rank: Rank = Rank.NONE
    """Liturgical rank of the celebration."""

    @staticmethod
    def parse(data: ApiEvent) -> "Event":
        """Parses the data from the API into an Event object.

        Args:
            data (ApiEvent): The data from the API.

        Returns:
            Event: The event containing all data from the API.
        """
        title = str(data.get("Tl"))
        colors = {
            "w": Color.WHITE,
            "r": Color.RED,
            "g": Color.GREEN,
            "v": Color.VIOLET,
            "": Color.NONE,
        }
        color = colors.get(str(data.get("Farbe")))
        rose_days = ["Dritter Adventssonntag", "Vierter Fastensonntag"]
        if title in rose_days:
            color = Color.ROSE
        ranks = {
            "H": Rank.SOLEMNITY,
            "F": Rank.FEAST,
            "G": Rank.MEMORIAL,
            "g": Rank.OPTIONAL_MEMORIAL,
            "Kommemoration": Rank.COMMEMORATION,
        }
        rank = ranks.get(str(data.get("Rang")), Rank.NONE)

        return Event(
            date=date.fromisoformat(str(data.get("Datum"))),
            title=title,
            comment=str(data.get("Bem")),
            lecture1=str(data.get("L1")),
            psalm=str(data.get("AP")),
            lecture2=str(data.get("L2")),
            gospel=str(data.get("EV")),
            color=color,
            file=str(data.get("Datei")),
            importance=int(data.get("Grad", 0)),
            rank=rank,
        )



"""
Simple utility module for reused functions that don't fit into any other module.
"""

from datetime import date, datetime


def normalize_date(year: date | datetime | int, month: int = 0, day: int = 0) -> date:
    """Returns a date object from a date or datetime object or even numbers.

    If a date or datetime object is passed, it is stripped of time information.
    If only a year as number is passed, the month and day numbers are extracted
    from the other arguments.

    Args:
        year (int | date | datetime): Either a date to check or the year if
            month and day are also provided.
        month (int, optional): If the year is a number, the month must also
            be provided. Defaults to 0.
        day (int, optional): If the year is a number, the day must also be
            provided. Defaults to 0.

    Returns:
        The normalized date object, stripped of time information.
    """
    if isinstance(year, datetime):
        return year.date()
    if isinstance(year, date):
        return year
    return date(year, month, day)

def easter(year: int | date | datetime) -> date:
    """Calculates the date of Easter Sunday for a given year.

    If a date or datetime object is passed, the year is extracted from it.

    Args:
        year: The year for which to calculate Easter Sunday.

    Returns:
        The date of Easter Sunday for the given year.
    """
    if isinstance(year, datetime) or isinstance(year, date):
        year = year.year
    a, b, c = year % 19, year // 100, year % 100
    d = (19 * a + b - b // 4 - ((b - (b + 8) // 25 + 1) // 3) + 15) % 30
    e = (32 + 2 * (b % 4) + 2 * (c // 4) - d - (c % 4)) % 7
    f = d + e - 7 * ((a + 11 * d + 22 * e) // 451) + 114
    return date(year, f // 31, f % 31 + 1)



"""
Module containing different approaches on handling API data.

The idea is to have a common interface for different backends, that provide the
information of the API of http://www.eucharistiefeier.de. It is organized by
years so that the data may be cached.
"""

import json
import os
from abc import ABC, abstractmethod
from datetime import date, datetime
from typing import Any, Dict, List, Mapping, cast
from urllib.parse import urlencode

import requests
from platformdirs import user_data_dir

from . import utils

ApiEvent = Mapping[str, str | int]
"""Type alias for the event data returned by the API."""

ApiYear = Mapping[str, str | List[ApiEvent]]
"""Type alias for a whole year of events returned by the API."""


class Api(ABC):
    """An abstract class representing an API response.

    To simplify the implementation of various backends, this abstract class is
    used to define a common interface. The implementing classes should return
    the JSON-data returned by the API of http://www.eucharistiefeier.de.
    """

    @abstractmethod
    def get_year(self, year: int) -> ApiYear:
        """Should return the JSON-data for a given year.

        Args:
            year (int): The year for which to return the JSON-data.

        Returns:
            ApiYear: The JSON-data as provided by the API described above.
        """
        pass

    def get_date(
        self, year: int | date | datetime, month: int = 0, day: int = 0,
    ) -> List[ApiEvent]:
        """Returns a list of data for a given date.

        The data will be retrieved from the whole year of data provided.

        Args:
            year (int | date | datetime): Either a date to check or the year if
                month and day are also provided.
            month (int, optional): If the year is a number, the month must also
                be provided. Defaults to 0.
            day (int, optional): If the year is a number, the day must also be
                provided. Defaults to 0.

        Returns:
            List[ApiEvent]: As multiple events may be on the same day, a list is
                returned.
        """
        date = utils.normalize_date(year, month, day)
        data = self.get_year(date.year)["Zelebrationen"]
        data = cast(List[ApiEvent], data)
        date_param = date.strftime("%Y-%m-%d")
        return [d for d in data if d["Datum"] == date_param]


class RequestApi(Api):
    """An API using the requests library to fetch the data directly.

    This class directly trys to fetch all requested data directly from the web
    API. It doesn't implement any caching or file handling.

    Attributes:
        calendar (str | None): The calendar to use. If None, the default
            calendar is used. See the `kal`-parameter of the API.
    """

    def __init__(self, calendar: str | None = None):
        """Creates an API-handler.

        Args:
            calendar (str | None, optional): The regional calendar to use.
                Defaults to None.
        """
        self.calendar = calendar

    def _get_params(self, year: int) -> Dict[str, str]:
        """Internal method to prepare common parameters for the API.

        Args:
            year (int): The year to request.

        Returns:
            Dict[str, str]: Parameters to send as GET-parameters to the API.
        """
        return {
            "format": "jsonarray",
            "jahr": str(year),
            "info": "wdtrgflu",
            "dup": "e",
            "bahn": "j",
            "kal": self.calendar if self.calendar else "",
        }

    def _request(self, params: Dict[str, str]) -> Dict[str, Any]:
        """Internal method to request data from the API.

        Args:
            params (Dict[str, str]): The parameters to add as query parameters.

        Returns:
            Dict[str, Any]: The response of the API as a dictionary.
        """
        url = f"http://www.eucharistiefeier.de/lk/api.php?{urlencode(params)}"
        return requests.get(url, timeout=10).json()

    def get_year(self, year: int) -> ApiYear:
        """Returns the JSON-data for a given year.

        Args:
            year (int): The year for which to return the JSON-data.

        Returns:
            ApiYear: The JSON-data freshly requested from the API.
        """
        params = self._get_params(year)
        return self._request(params)

    def get_date(
        self, year: int | date | datetime, month: int = 0, day: int = 0,
    ) -> List[ApiEvent]:
        """Returns a list of data for a given date.

        This method is overwritten as to only request the data for the given
        date from the API instead of the whole year.

        Args:
            year (int | date | datetime): Either a date to check or the year if
                month and day are also provided.
            month (int, optional): If the year is a number, the month must also
                be provided. Defaults to 0.
            day (int, optional): If the year is a number, the day must also be
                provided. Defaults to 0.

        Returns:
            List[ApiEvent]: As multiple events may be on the same day, a list is
                returned.
        """
        date = utils.normalize_date(year, month, day)
        params = self._get_params(date.year)
        params["monat"] = str(date.month)
        params["tag"] = str(date.day)
        data = self._request(params)
        return data["Zelebrationen"]

    def get_date_from_year(
        self, year: int | date | datetime, month: int = 0, day: int = 0,
    ) -> List[ApiEvent]:
        """Returns a list of data for a given date.

        The data will be retrieved from the whole year of data provided. Useful
        if you want to enforce the use of the whole year of data. E.g. if the
        `get_year`-method is overwritten to use a cache.

        Args:
            year (int | date | datetime): Either a date to check or the year if
                month and day are also provided.
            month (int, optional): If the year is a number, the month must also
                be provided. Defaults to 0.
            day (int, optional): If the year is a number, the day must also be
                provided. Defaults to 0.

        Returns:
            List[ApiEvent]: As multiple events may be on the same day, a list is
                returned.
        """
        return super().get_date(year, month, day)


class FileApi(Api):
    """An api that reads the data from a file.

    This backend may be useful if you don't want to use an online API. This
    requires you to firstly download the data from the API and save it to a
    file. The file must contain the JSON-data of a whole year of events. An
    example code to create this files could be:

    ```python
    years = range(2020, 2026)
    api = RequestApi()
    for year in years:
        data = api.get_year(year)
        with open(f"path/{year}.json", "w") as f:
            json.dump(data, f)
    ```

    Attributes:
        format_path (str): A format string that is used to load the data for a
            given year. The year will be inserted into the string using the
            `%`-operator.
    """

    def __init__(self, format_path: str):
        """Creates the API-handler.

        Args:
            format_path (str): A format string that is used to load the data for
                a given year. The year will be inserted into the string using the
                `%`-operator.
        """
        self.format_path = format_path

    def get_year(self, year: int) -> ApiYear:
        """Returns the JSON-data for a given year.

        Args:
            year (int): The year for which to return the JSON-data.

        Returns:
            ApiYear: The JSON-data loaded from the file.
        """
        path = self.format_path % year
        with open(path, "r") as f:
            return json.load(f)


class CacheApi(RequestApi):
    """An API extension that caches requested data to a file.

    Additionally to the `RequestApi`, this class saves the data to a file after
    being loaded. This way, the data is only requested once and then saved to a
    file. If the data is requested again, it is loaded from the file instead of
    the API.

    Attributes:
        base_path (str): The base path where the files are saved. In the given
            directory, multiple JSON-files will be saved.
        calendar (str | None): The calendar to use. If None, the default
            calendar is used. See the `kal`-parameter of the API.
    """

    def __init__(self, base_path: str | None = None, calendar: str | None = None):
        """Creates an API-handler.

        Args:
            base_path (str | None, optional): The base path where the files are
                saved. In the given directory, multiple JSON-files will be
                created. If no path is given, a temporary directory according to
                the operating system is used. Defaults to None.
            calendar (str | None, optional): The regional calendar to use.
                Defaults to None.
        """
        super().__init__(calendar)
        if base_path is None:
            base_path = user_data_dir("directorium", False, ensure_exists=True)
        self.base_path = base_path

    def get_year(self, year: int) -> ApiYear:
        """Returns the JSON-data for a given year.

        Args:
            year (int): The year for which to return the JSON-data.

        Returns:
            ApiYear: The JSON-data out of the cached data or requested from the
                API if no local data is available.
        """
        cal_name = f".{self.calendar}" if self.calendar else ""
        path = f"{self.base_path}/{year}{cal_name}.json"

        if os.path.exists(path):
            with open(path, "r") as f:
                return json.load(f)

        data = super().get_year(year)
        with open(path, "w") as f:
            json.dump(data, f)
        return data

    def get_date(
        self, year: int | date | datetime, month: int = 0, day: int = 0,
    ) -> List[ApiEvent]:
        """Returns a list of data for a given date.

        The data will be retrieved from the whole year of data provided. This
        way caching of the yearly data is possible.

        Args:
            year (int | date | datetime): Either a date to check or the year if
                month and day are also provided.
            month (int, optional): If the year is a number, the month must also
                be provided. Defaults to 0.
            day (int, optional): If the year is a number, the day must also be
                provided. Defaults to 0.

        Returns:
            List[ApiEvent]: As multiple events may be on the same day, a list is
                returned.
        """
        return super().get_date_from_year(year, month, day)



import json
from datetime import date
from unittest import TestCase

from directorium import Directorium
from directorium.api import CacheApi, FileApi, RequestApi
from directorium.directorium import Season
from directorium.event import Event


class TestDirectorium(TestCase):

    def setUp(self):
        with open("tests/data/2024.json", "r") as f:
            self.data = json.load(f)["Zelebrationen"]
        self.api = FileApi("tests/data/%s.json")

    def test_get(self):
        directorium = Directorium(self.api)
        expected = [Event.parse(self.data[0])]
        actual = directorium.get(date(2024, 1, 1))
        self.assertEqual(expected, actual)

    def test_get_multiple(self):
        directorium = Directorium(self.api)
        expected = [Event.parse(d) for d in self.data[1:3]]
        actual = directorium.get(date(2024, 1, 2))
        self.assertEqual(expected, actual)

    def test_season(self):
        directorium = Directorium(self.api)
        self.assertEqual(directorium.season(date(2024, 1, 1)), Season.CHRISTMAS)
        self.assertEqual(directorium.season(date(2024, 2, 5)), Season.ORDINARY)
        self.assertEqual(directorium.season(date(2024, 3, 21)), Season.LENT)
        self.assertEqual(directorium.season(date(2024, 4, 1)), Season.EASTER)

    def test_factory_methods(self):
        directorium = Directorium.from_request("koeln")
        self.assertIsInstance(directorium.api, RequestApi)
        self.assertEqual(directorium.api.calendar, "koeln")
        directorium = Directorium.from_file("tests/data/%s.json")
        self.assertIsInstance(directorium.api, FileApi)
        self.assertEqual(directorium.api.format_path, "tests/data/%s.json")
        directorium = Directorium.from_cache("tests/data/", "koeln")
        self.assertIsInstance(directorium.api, CacheApi)
        self.assertEqual(directorium.api.base_path, "tests/data/")
        self.assertEqual(directorium.api.calendar, "koeln")



from datetime import date, datetime
from unittest import TestCase

from directorium import utils


class TestUtils(TestCase):

    easter_dates = [
        date(2015, 4, 5),
        date(2016, 3, 27),
        date(2017, 4, 16),
        date(2018, 4, 1),
        date(2019, 4, 21),
        date(2020, 4, 12),
        date(2021, 4, 4),
        date(2022, 4, 17),
        date(2023, 4, 9),
        date(2024, 3, 31),
        date(2025, 4, 20),
        date(2026, 4, 5),
        date(2027, 3, 28),
        date(2028, 4, 16),
        date(2029, 4, 1),
        date(2030, 4, 21),
    ]

    def test_easter_date_for_year_number(self):
        for d in self.easter_dates:
            self.assertEqual(utils.easter(d.year), d)

    def test_easter_date_for_date(self):
        for d in self.easter_dates:
            self.assertEqual(utils.easter(date(d.year, 3, 1)), d)

    def test_easter_date_for_datetime(self):
        for d in self.easter_dates:
            self.assertEqual(utils.easter(datetime(d.year, 3, 1, 12, 3, 5)), d)

    def test_normalize_date(self):
        self.assertEqual(utils.normalize_date(date(2020, 2, 29)), date(2020, 2, 29))
        self.assertEqual(utils.normalize_date(datetime(2020, 2, 29, 12, 3, 5)), date(2020, 2, 29))
        self.assertEqual(utils.normalize_date(2020, 5, 23), date(2020, 5, 23))
        with self.assertRaises(ValueError):
            utils.normalize_date(2020)
        with self.assertRaises(ValueError):
            utils.normalize_date(2020, 5)



import json
from datetime import date
from unittest import TestCase

from directorium.event import Color, Event, Rank


class TestEvent(TestCase):

    def setUp(self):
        with open("tests/data/2024.json", "r") as f:
            self.data = json.load(f)["Zelebrationen"]

    def test_parse(self):
        expected = Event(
            date=date(2024,1,1),
            title="1. Januar - Neujahr",
            comment="Hochfest der Gottesmutter Maria",
            lecture1="Num 6, 22-27",
            psalm="Ps 67 (66), 2-3.5.6 u. 8 (R: 2a)",
            lecture2="Gal 4, 4-7",
            gospel="Lk 2, 16-21",
            color=Color.WHITE,
            file="weihnachtszeit/oktavtag.htm",
            importance=3,
            rank=Rank.SOLEMNITY,
        )
        actual = Event.parse(self.data[0])
        self.assertEqual(actual, expected)

    def test_parse_pink_color(self):
        days = ["Dritter Adventssonntag", "Vierter Fastensonntag"]
        data = [d for d in self.data if d["Tl"] in days]
        for d in data:
            actual = Event.parse(d)
            self.assertEqual(actual.color, Color.ROSE)
